(* ::Package:: *)

(* ::Title:: *)
(*GData*)


(* ::Author:: *)
(*Rob Raguet-Schofield*)
(*packages@ragfield.com*)


(* ::Section:: *)
(*Initialization*)


(* ::Input:: *)
(*Needs["JLink`"];*)


BeginPackage["GData`", {"JLink`"}];


YouTube::usage = "YouTube[<...>] is an opaque object containing session information for a connection to YouTube.";
YouTubeLogin::usage = "YouTubeLogin[(\"User\"->user (, \"Password\"->pass)) logs into YouTube and returns a YouTube session object for use in other functions";
YouTubeLogout::usage = "YouTubeLogout[yt] logs out of YouTube and destroys the YouTube session object";
YouTubeUploads::usage = "YouTubeUploads[yt (, user)] returns a list of rules describing the videos uploaded by the specified user (or the currently logged in user)";
YouTubeFavorites::usage = "YouTubeFavorites[yt (,user)] returns a list of rules describing the favorite videos of the specified user (or the currently logged in user)";
YouTubeUpload::usage = "YouTubeUpload[yt, filePath (, \"Title\"->title, \"Description\"->desc, \"Category\"->cat, \"Keywords\"->{...})] uploads a video file to YouTube";


Begin["`Private`"];


(* ::Section:: *)
(*Implementation*)


(* ::Subsection:: *)
(*Common*)


Options[GDataLogin] = {
	"User" -> "",
	"Password" -> ""
};

GDataLogin[service_String, opts___?OptionQ] := Module[
	{user, pass, res = True},
	user = "User" /. {opts} /. Options[GDataLogin];
	pass = "Password" /. {opts} /. Options[GDataLogin];
	
	If[StringLength[user] === 0 || StringLength[pass] === 0,
		DynamicModule[{dynUser = user, dynPass = pass},
			res = ChoiceDialog[Column[{
				TextCell["Login to " <> service,
					FontSize->CurrentValue[{"ControlsFontSize", Large}]],
				TextCell["User name:"],
				InputField[Dynamic[dynUser], String],
				TextCell["Password:"],
				InputField[Dynamic[dynPass], String,
					BaseStyle->{FontOpacity->0}]
			}]];
			user = dynUser;
			pass = dynPass;
		]
	];
	If[!res, pass = $Canceled; user = $Canceled];
	{user, pass}
];


(* ::Subsection:: *)
(*YouTube*)


LoadJavaClass["java.lang.Class"];
LoadJavaClass["com.google.gdata.data.youtube.YouTubeNamespace"];


$YouTubeKey = "AI39si5dkxS5zv3skaq0Es9muI_bqFWf4_2PiBRH02kM_uh3UU62MCiAIc2rLBpix7jGTk-CaHHvf2y7nTnwXyhViWS83RGxaA";


$YouTubeServer = "http://gdata.youtube.com";


$YouTubeUploadServer = "http://uploads.gdata.youtube.com";


$YouTubeUserFeed = StringJoin[$YouTubeServer, "/feeds/api/users/"];


$YouTubeUserUploadFeed = StringJoin[$YouTubeUploadServer, "/feeds/api/users/"];


Format[yt_YouTube, StandardForm] := "YouTube[<...>]";


YouTubeUser[yt_YouTube] := Part[yt, 1];


YouTubeService[yt_YouTube] := Part[yt, 2];


YouTubeUserFeed[user_String, forUpload:(True|False)] :=
	StringJoin[If[forUpload,
		$YouTubeUserUploadFeed,
		$YouTubeUserFeed],
	user];


YouTubeUserFeed[yt_YouTube, forUpload:(True|False)] :=
	YouTubeUserFeed[YouTubeUser[yt], forUpload];


Options[YouTubeLogin] = Options[GDataLogin];

YouTubeLogin[opts___?OptionQ] := Module[
	{user, pass, service},
	user = "User" /. {opts} /. Options[YouTubeLogin];
	pass = "Password" /. {opts} /. Options[YouTubeLogin];
	{user, pass} = GDataLogin["YouTube", "User"->user, "Password"->pass];
	If[user==$Canceled || pass==$Canceled, Return[$Canceled]];
	service = JavaBlock[
		service = JavaNew["com.google.gdata.client.youtube.YouTubeService",
			"Mathematica-GData-YouTube", $YouTubeKey];
		Block[
			{$JavaExceptionHandler = (service=Null)&},
			service@setUserCredentials[user, pass]
		];
		service
	];
	If[service===Null, $Failed, YouTube[user, service]]
];


(* ::Input:: *)
(*yt=YouTubeLogin["User"->"robragfield"]*)


YouTubeLogout[yt_YouTube] := ReleaseJavaObject[YouTubeService[yt]];


YouTubeEntry[entry_] := Module[
	{mg, mp, stats},
	mg = entry@getMediaGroup[];
	mp = mg@getPlayer[];
	stats = entry@getStatistics[];
	{
		"Title" -> entry@getTitle[]@getPlainText[],
		"Private" -> (mg =!= Null && mg@isPrivate[]),
		"URL" -> If[mg =!= Null, mp@getUrl[], Null],
		"FavoriteCount" -> If[stats =!= Null, stats@getFavoriteCount[], Missing["Unknown"]],
		"ViewCount" -> If[stats =!= Null, stats@getViewCount[], Missing["Unknown"]]
	}
];


YouTubeFeed[feed_] := Module[
	{iter, entries = {}},
	iter = feed@getEntries[]@iterator[];
	While[iter@hasNext[],
		AppendTo[entries, YouTubeEntry[iter@next[]]]
	];
	entries
];


YouTubeFeed[yt_YouTube, url_String] := Module[
	{u = url, feed, iter, entry, mg, mp, entries = {}},
	JavaBlock[
		While[u =!= Null,
			feed = YouTubeService[yt]@getFeed[
				JavaNew["java.net.URL", u],
				Class`forName["com.google.gdata.data.youtube.VideoFeed",
					True, YouTubeService[yt]@getClass[]@getClassLoader[]
				]
			];
			entries = Join[entries, YouTubeFeed[feed]];
			u = If[feed@getNextLink[] =!= Null,
				feed@getNextLink[]@getHref[], Null];
		];
		entries
	]
];


YouTubeUploads[yt_YouTube] :=
	YouTubeUploads[yt, YouTubeUser[yt]];

YouTubeUploads[yt_YouTube, user_String] :=
	YouTubeFeed[yt, YouTubeUserFeed[user, False]<>"/uploads"];




(* ::Input:: *)
(*YouTubeUploads[yt]*)


(* ::Input:: *)
(*YouTubeUploads[yt, "wolframresearch"]//Length*)


(* ::Input:: *)
(*YouTubeUploads[yt, "wolframmathematica"]//Length*)


YouTubeFavorites[yt_YouTube] :=
	YouTubeFavorites[yt, YouTubeUser[yt]];

YouTubeFavorites[yt_YouTube, user_String] :=
	YouTubeFeed[yt, YouTubeUserFeed[user, False]<>"/favorites"];


(* ::Input:: *)
(*YouTubeFavorites[yt]*)


(* ::Input:: *)
(*YouTubeFavorites[yt,"wolframresearch"]*)


(* ::Input:: *)
(*YouTubeFavorites[yt,"wolframmathematica"]*)


Options[YouTubeUpload] = {
	"Title" -> Automatic,
	"Description" -> "Undescripted",
	"Category" -> "Tech",
	"Keywords" -> {"uploaded by Mathematica"}
};

YouTubeUpload[yt_YouTube, filePath_String, opts___?OptionQ] := Module[
	{file, entry, mg, title, cat, keywords, desc, ms, uploadURL, res},
	JavaBlock[
		file = JavaNew["java.io.File", filePath];
		entry = JavaNew["com.google.gdata.data.youtube.VideoEntry"];
		
		mg = entry@getOrCreateMediaGroup[];
		
		title = ("Title" /. {opts} /. Options[YouTubeUpload]) /.
			{Automatic -> FileNameTake[filePath, -1]};
		mg@setTitle[JavaNew["com.google.gdata.data.media.mediarss.MediaTitle"]];
		mg@getTitle[]@setPlainTextContent[title];
		
		cat = "Category" /. {opts} /. Options[YouTubeUpload];
		cat = JavaNew["com.google.gdata.data.media.mediarss.MediaCategory",
			YouTubeNamespace`CATEGORYUSCHEME, cat];
		mg@addCategory[cat];
		
		keywords = "Keywords" /. {opts} /. Options[YouTubeUpload];
		mg@setKeywords[JavaNew["com.google.gdata.data.media.mediarss.MediaKeywords"]];
		(mg@getKeywords[]@addKeyword[#])& /@ keywords;
		
		desc = "Description" /. {opts} /. Options[YouTubeUpload];
		mg@setDescription[JavaNew["com.google.gdata.data.media.mediarss.MediaDescription"]];
		mg@getDescription[]@setPlainTextContent[desc];
		
		ms = JavaNew["com.google.gdata.data.media.MediaFileSource",
			file, "application/octet-stream"];
		entry@setMediaSource[ms];
		
		uploadURL = JavaNew["java.net.URL",
			YouTubeUserFeed[YouTubeUser[yt], True]<>"/uploads"];
		
		entry = YouTubeService[yt]@insert[uploadURL, entry];
		If[entry === $Failed, $Failed, YouTubeEntry[entry]]
	]
];


(* ::Input:: *)
(*YouTubeUpload[yt,"/Users/schofield/Movies/OmetepeProfile.mov"]*)


(* ::Input:: *)
(*LoadJavaClass["java.lang.System"]*)


(* ::Input:: *)
(*System`setProperty["http.proxyHost", "proxy.wolfram.com"]*)
(*System`setProperty["http.proxyPort", "8080"]*)
(*System`setProperty["https.proxyHost", "proxy.wolfram.com"]*)
(*System`setProperty["https.proxyPort", "8080"]*)
(*System`setProperty["socksProxyHost", "socks.wolfram.com"]*)
(*System`setProperty["socksProxyPort", "1080"]*)


(* ::Section:: *)
(*Finalization*)


End[]; (* `Private` *)


EndPackage[]; (* GData` *)
